@@ -22,7 +22,7 @@ from dagster._utils.merger import deep_merge_dicts
 from pydantic import Field
 from requests.exceptions import RequestException
 
-from .types import DbtCloudOutput
+from dagster_dbt.cloud.types import DbtCloudOutput
 
 DBT_DEFAULT_HOST = "https://cloud.getdbt.com/"
 DBT_API_V2_PATH = "api/v2/accounts/"
@@ -470,10 +470,8 @@ class DbtCloudClient:
                 ):
                     self.cancel_run(run_id)
                     raise Failure(
-                        (
-                            f"Run {run_id} timed out after "
-                            f"{datetime.datetime.now() - poll_start}. Attempted to cancel."
-                        ),
+                        f"Run {run_id} timed out after "
+                        f"{datetime.datetime.now() - poll_start}. Attempted to cancel.",
                         metadata={"run_page_url": MetadataValue.url(href)},
                     )
 
@@ -592,10 +590,6 @@ class DbtCloudResource(DbtCloudClient):
     pass
 
 
-# This is a temporary shim to support the old resource name.
-DbtCloudResourceV2 = DbtCloudResource
-
-
 class DbtCloudClientResource(ConfigurableResource, IAttachDifferentObjectToOpContext):
     """This resource helps interact with dbt Cloud connectors."""
 
